﻿using C1.WPF;
using C1.WPF.Core;
using C1.WPF.Input;
using C1.WPF.PropertyGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PropertyGrid20220429
{

    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();


            List<FontFamily> fontList = new List<FontFamily>();
            foreach (System.Drawing.FontFamily font in System.Drawing.FontFamily.Families)
            {
                fontList.Add(new FontFamily(font.Name));
            }

            PropertyAttribute propertyAttribute = new PropertyAttribute();
            propertyAttribute.MemberName = "FontFamily";
            propertyAttribute.DisplayName = "FontFamily";
            propertyAttribute.Category = "Font Category";

            propertyAttribute.Editor = new CustomEditor(fontList);
            propertyGrid.PropertyAttributes.Add(propertyAttribute);

            propertyGrid.SelectedObject = txtBox;
        }

        private void Test(object sender, RoutedEventArgs e)
        {
            txtBox.FontFamily = new FontFamily("Times New Roman");//update
        }
    }

    public class CustomEditor : C1ComboBox, ITypeEditorControl
    {
        // Events
        public event PropertyChangedEventHandler ValueChanged;

        // Fields
        internal PropertyAttribute _selectedProperty;
        private List<FontFamily> _itemsSource;

        // Implementation
        public CustomEditor(List<FontFamily> itemsSource)
        {
            _itemsSource = itemsSource;
        }

        public ITypeEditorControl Create()
        {
            return new CustomEditor(_itemsSource);
        }

        public void Attach(PropertyAttribute property)
        {
            // Store current property
            _selectedProperty = property;

            // Set ComboBox
            this.ItemsSource = _itemsSource;
            this.SelectedItemChanged += CustomEditor_SelectedItemChanged;

            Binding editorBinding = new Binding("SelectedItem") { TargetNullValue = new FontFamily("") };
            editorBinding.Source = this;
            editorBinding.Mode = BindingMode.TwoWay;
            BindingOperations.SetBinding(property.SelectedObject as TextBox, TextBox.FontFamilyProperty, editorBinding);

            if (this.SelectedItem == null)
                (property.SelectedObject as TextBox).FontFamily = new FontFamily("Arial");//default      
        }

        public void Detach(PropertyAttribute property)
        {
            this.SelectedItemChanged -= CustomEditor_SelectedItemChanged;
        }

        public bool Supports(PropertyAttribute Property)
        {
            return Property.PropertyInfo.PropertyType == typeof(string);
        }

        private void CustomEditor_SelectedItemChanged(object sender, PropertyChangedEventArgs<object> e)
        {
            if (ValueChanged != null)
                ValueChanged(this, new PropertyChangedEventArgs(_selectedProperty.MemberName));
        }
    }
}
